/*
 * dat.h
 *
 * $Source: /usr/people/dpwe/src/dat/lib/dat/RCS/dat.h,v $
 * $Date: 94/12/13 20:42:09 $
 * $Author: dpwe $
 * $Locker:  $
 *
 * This file is part of the dat library.
 *
 * It contains definitions for dat file utilities, data structures, and
 * standard keywords used by dat.
 *
 * $Log:	dat.h,v $
 * Revision 1.21  94/12/13  20:42:09  dpwe
 * dpwe: added SAMPLERATE header key definition
 * 
 * Revision 1.20  94/08/16  20:16:27  sbeck
 * added character datatype abbreviations
 * 
 * Revision 1.19  1994/01/07  22:26:18  sbeck
 * Removed all evidence of floating point representation
 * since it is now completely handled by proper byte order
 * processing. Also removed architecture specific definition
 * of the macro MACH_BYTE_ORDER since the the function call
 * mach_get_byte_order() now can be used to automatically
 * determine the endian-ness of the host machine.
 *
 * I've decided to not define MACH_BYTE_ORDER as the function
 * mach_get_byte_order() to emphasize that it is a function call, and
 * thus expends processing effort, rather than just being a constant.
 *
 * Revision 1.18  1993/12/17  16:12:06  sbeck
 * macro BYTE_ORDER changed to macro BYTE_ORDER_KEY due to
 * conflict under alpha osf 1's macro of same name in
 * /usr/include/machine/endian.h
 * -Shawn
 *
 * Revision 1.17  1993/10/14  20:02:29  giri
 * Added Machine order for i860
 *
 * Revision 1.16  1993/06/17  11:33:03  lacsap
 * Added byte order frob for NeXT fat binary
 * -lacsap
 *
 * Revision 1.15  93/06/16  20:23:19  holtzman
 * Added int32 datatype which is an int on alpha's and a long otherwise.
 * -Henry
 * 
 * Revision 1.14  1993/04/12  23:08:12  holtzman
 * set the byte oreder and float rep for alphas
 *
 * Revision 1.13  1993/03/16  16:06:11  holtzman
 * Whoever replaced the dependancy on __STDC__ with __cplusplus is
 * a bozo, woja.  I put back the dependancy on __STDC__, but out of
 * consideration, I also added the logic for looking for both.
 *
 * Revision 1.12  93/02/24  15:47:25  woja
 * Added conditional for c++ support (full args for fn definitions)
 * 
 * Revision 1.11  93/02/19  23:41:34  holtzman
 * Changed the floating point type of a MIPSEB machine to IEEE_FLOAT,
 * cause that's what it is.  MIPSEL (decstations) are IEEE_FLOAT too,
 * but I didn't change that because of the 100s of improperly tagged
 * datfiles out there.  -Henry
 * 
 * Revision 1.10  93/02/19  23:30:29  holtzman
 * Found lying around.  Looks like woja made some changes.
 * 
 * Revision 1.9  22/.1/.2  .0:.5:.4  holtzman
 * Got rid of NOIOBUF.  Should use FILE instead of struct _iobuf
 * 
 * Revision 1.8  22/.1/.2  .2:.5:.4  holtzman
 * fixed declaration for d_set_float_rep
 * 
 * Revision 1.7  22/.1/.2  .2:.5:.4  holtzman
 * *** empty log message ***
 * 
 * Revision 1.6  91/11/02  23:31:32  klee
 * updated for const declarations
 * 
 * Revision 1.5  91/10/20  15:04:14  klee
 * Unknown changes.
 * 
 * Revision 1.5  90/07/18  03:19:53  pizza
 * Added the checking for float and endian architecture flags. Put in
 * machine-dependent float and endian macros.
 *
 * Revision 1.4  88/03/18  16:50:51  steve
 * Add D_ADDR_DSET and D_ADDR_CHAN defs.
 *
 * Revision 1.3  86/03/07  00:36:55  steve
 * Added datio definitions.
 *
 * Revision 1.2  85/06/30  17:07:58  musicus
 * Added d_settype() definition
 *
 * Revision 1.1  85/06/28  23:48:18  musicus
 * Initial revision
 *
 *      Edit history: complete, wdh, 3/19/83
 *      Total rewrite: musicus 7/11/83
 *      Another rewrite: musicus 10/24/83
 *      Changed macro definitions, musicus 5/29/84
 */

#ifndef	_DATH_                            
#define	_DATH_

#include <stdio.h>                       

#ifndef TRUE
#define TRUE (1)
#endif

#ifndef FALSE
#define FALSE (0)
#endif

#ifndef int32
#if defined(__alpha)
#define int32 int
#else
#define int32 long
#endif
#endif

struct d_token
{
  struct d_token *d_next;
  char *d_key;
  char *d_value;
};

typedef struct _db DATFILE;

 struct _db
{
  struct d_token *db_h;	/* token tree ptr */
  FILE *db_fp;		/* dat file pointer */
  char *db_name;	/* data base name */
  char db_flag;		/* flag of dat file status */
  char db_nbs;		/* number of bits to shift sample count left
			   to get byte count */
};

#define _DB_WRHDR	001	/* dat file header modified */

/* 
 * Keywords for dat file descriptor
 */

/* type of data in file -- required */
#define	TYPE		"_data_type"

/* dimensions of each data set -- required */
#define	DIMENSIONS	"_dimensions"

/* number of bits in each bit field -- required for type bit_field */
#define FIELD_SIZE	"_field_size"

/* optional - required for port. big or little endian */
#define BYTE_ORDER_KEY	"_byte_order"

/* optional - arg is number of datasets in file - default=1 */
#define	DATASETS	"_data_sets"

/* optional - arg is number of channels in each record */
#define	CHANNELS	"_channels"

/* optional */
#define	COMMENT		"_comment"

/* optional - should be appended by every command which changes the dat file */
#define	HISTORY		"_history"

/* optional - sampling period; a number, followed by units */
#define	PERIOD		"_sampling_period"

/* dpwe@media.mit.edu : extension : sampling rate */
#define	SAMPLERATE	"_sample_rate"


/*
 * Values for dat file descriptor keywords.
 * comments to the right show the corresponding type character 
 * used by istype(), d_{set,get}type and d_{read,write}[mc] 
 */

/* TYPE */
#define	BITFIELD	"bit_field"
#define	ASCII		"ascii"			/* 'a' */
#define	FLOAT		"float"			/* 'f' */
#define	DOUBLE		"double"		/* 'F' */
#define	COMPLEX		"complex"		/* 'z' */
#define	DCOMPLEX	"double_complex"	/* 'Z' */
#define	INTEGER1	"signed_1"		/* 'c' */
#define	INTEGER2	"signed_2"		/* 'd' */
#define	INTEGER4	"signed_4"		/* 'D' */
#define	UNSIGNED1	"unsigned_1"		/* 'b' */
#define	UNSIGNED2	"unsigned_2"		/* 'u' */
#define	UNSIGNED4	"unsigned_4"		/* 'U' */
#define	FIX2		"fixed_point_2"		/* 'x' */
#define	FIX4		"fixed_point_4"		/* 'X' */

/* DIMENSION or TYPE */
#define	VARIANT		"variant"

/* ENDIAN */
enum byte_order {MSB, LSB} ;

/* NOKEY must be defined to some value that is not enumerated above */
#define NOKEY -1

#ifdef USE_MACH_ENDIAN
/* 
 * define machine specific dependencies 
 * the default is -
 * 	(_byte_order msb)
 * 
 * only if a paticular machine deviates from the default should
 * anything be added here.
 * 
 * default machines:
 * sparc, sun3, i386, NeXT, sun4
 *
 * others:
 * vax, mips
 */
#if defined vax
#define MACH_BYTE_ORDER	LSB
#elif defined MIPSEL
#define MACH_BYTE_ORDER	LSB
#elif defined MIPSEB
#define MACH_BYTE_ORDER MSB
#elif defined __alpha
#define MACH_BYTE_ORDER LSB
#elif defined i860
#define MACH_BYTE_ORDER LSB
#elif defined NeXT
#ifdef __BIG_ENDIAN__
#define MACH_BYTE_ORDER MSB
#else
#define MACH_BYTE_ORDER LSB
#endif
#endif
#ifndef MACH_BYTE_ORDER
#define MACH_BYTE_ORDER	MSB
#endif
#endif /*USE_MACH_ENDIAN*/

/* macros for dat file package */
#define d_getfp(d)	((d)->db_fp)
#define d_getname(d)	((d)->db_name)


#define N_DIMENSIONS 10		/* max # of dimensions */
#define D_ADDR_CHAN (-1)	/* x.dim[D_ADDR_CHAN] equivalent to x.chan */
#define D_ADDR_DSET (-2)	/* x.dim[D_ADDR_DSET] equivalent to x.dset */

struct d_addr {		/* multi-dimensional address vector */
	long dset;		/* number of datasets */
	long chan;		/* number of channels */
	long dim[N_DIMENSIONS];	/* dimension array */
};

#if !defined(__cplusplus) && !defined(__STDC__)
DATFILE *d_open(), *d_open_subfile(), *d_descr(), *dxopen();
int d_close(),d_flush(),d_close_subfile();
char *d_getkey(), *d_getkey_elem();
int d_setkey(), d_appkey(), d_delkey(), d_appkey_strvec();
int d_gettype(), d_settype(), d_getdim(), d_setdim(), d_copyhdr();
int d_rmdat();
int scanargv(), getrange ();

int _dread(), _dmread(), _dwrite(), _dmwrite(),_dsize(),d_tell();
int samesize(), d_getsubdim();
long d_submatrix(), d_seek();

#else

DATFILE *d_open (const char *name, const char *mode);
DATFILE *d_open_subfile (DATFILE *df, const char *subfile, const char *mode);
DATFILE *d_descr (const char *name);
DATFILE *dxopen (const char *name, const char *mode);
int _dsize (int dtype);
int d_close (DATFILE *df);
int d_flush (DATFILE *df);
void _dclrinfo (int);
int d_close_subfile (DATFILE *df);
const char *d_getkey (const DATFILE *df, const char *key);
const char *d_getkey_elem (const DATFILE *df, const char *key, long idx);
int d_setkey (DATFILE *df, const char *key, const char *val);
int d_appkey (DATFILE *df, const char *key, const char *val);
int d_delkey (DATFILE *df, const char *key);
int d_appkey_strvec (DATFILE *df, const char *key, int vecsz, const char *const strvec[]);
int d_gettype (const DATFILE *df, int *nbps);
int d_settype (DATFILE *df, int type);
int d_getdim (const DATFILE *df, long *recs, long *chan, long dim[], int maxdim);
int d_setdim (DATFILE *df, long dset, long chan, long dim[], int ndim);
int d_copyhdr (DATFILE *to, const DATFILE *from);
int d_rmdat (const char *name);
int getrange (char *, char *, long *, long *, long, long *);
int scanargv(int, const char *const [], const char *, const char *, ...);
void subsample (char [], char [], int, int, int);
void convert_datatype (char [], int, char [], int, int);
void convert_endian(char [], int, int);
int _daterrh (char *, ...);
enum byte_order d_get_byte_order (DATFILE *);
enum byte_order mach_get_byte_order (void);
int d_set_byte_order (DATFILE *, enum byte_order);
int d_getsubmatrix(int, int *, int *, int *); 
int _dread(DATFILE *, char *, int, int);
int _dmread(DATFILE *, char **, int, int);
int _dwrite(DATFILE *, char *, int, int);
int _dmwrite(DATFILE *, char *, int ,int);
int d_tell(DATFILE *, struct d_addr *);
int samesize(DATFILE *, DATFILE *);
int d_getsubdim(DATFILE *, struct d_addr *);
long d_submatrix(DATFILE *, struct d_addr *, struct d_addr *, struct d_addr *);
long d_seek(DATFILE *, struct d_addr *, int);
#endif /* __STDC__ */

/* datfile I/O package */

#define d_read(df, buf, n)		_dread(df, buf, n, 0)
#define d_readc(df, buf, n, buftype)	_dread(df, buf, n, buftype)
#define d_readm(df, bufp, n)		_dmread(df, bufp, n, 0)
#define d_readcm(df, bufp, n, buftype)	_dmread(df, bufp, n, buftype)
#define d_write(df, buf, n)		_dwrite(df, buf, n, 0)
#define d_writec(df, buf, n, buftype)	_dwrite(df, buf, n, buftype)
#define d_writem(df, buf, n)		_dmwrite(df, buf, n, 0)
#define d_writecm(df, buf, n, buftype)	_dmwrite(df, buf, n, buftype)
#define daterrh                         _daterrh

#define istype(d1,dtype)		(d_gettype(d1,NULL)==dtype)
#define sametype(d1,d2)			(d_gettype(d1,NULL)==d_gettype(d2,NULL))
#define d_getsubmatrix d_getmatrix /* former name too long on some machines */

#endif /* _DATH_ */










