/***********************************************************************\
*	audIOlofi.c							*
*	New hardware-indep audio IO interface intended to be portable	*
*	based on lofiplay etc						*
*	dpwe 1993nov17, 13mar92						*
*	DECstation/DECaudio (LoFi) version - 1994apr07			*
\***********************************************************************/

#include <LoFi.h>

typedef struct {		/* descriptor for audio IO channel */
    int	mode;			/* AU_INPUT or AU_OUTPUT */
    int chans;			
    int format;
    void *port;
    long ourPos;
} AUSTRUCT;

#define AUSTRUCT_DEFINED	/* stop redefinition in audIO.h */

#include "audIO.h"	/* for stdio, PARG etc */

#include <assert.h>

#define DFLT_BUFTIME	(0.25)		/* default buffer length in secs */

#define NUM_PARAMS 1

/***************************************************************\
*	lofiplay.c (really newplay.c)				*
*	rewrite of lofiplay.c					*
*	maybe a bit clearer					*
*	dpwe 12may92 after lofiplay.c				*
\***************************************************************/

/* lofi memory map globals (from play*.asm code) */
#define ALIVEF	    0x0FF0	/* flag set when other regs ready */
#define SRATE	    0X0FF1	/* sampling rate requested */
#define HBSIZE	    0x0FF2	/* total size of host buffer (samples) */
#define HBBLCK	    0x0FF3	/* min block size by which HB advances */
#define HOBBASE	    0x0FF4	/* base of host output buffer */
#define HOBPNTR	    0x0FF5	/* current upper limit on host buffer */
#define HIBBASE	    0x0FF6	/* base of host input buffer */
#define HIBPNTR	    0x0FF7	/* upper limit on host buffer */

/* other constants */
#define MAXDBGWAIT	1000	/* how many iterations waiting for sync */
#define LOFI_CHANS	2	/* lofi buffer filled in pairs of samps */

/* static function prototypes */
static void WaitForLofiSampFrms PARG((int frames));
static void timeout PARG((char *s));
static int ReadDspPos PARG((void));
static void WaitForDspPosAt PARG((int ourpos, int posval));
static long RdDSPadr PARG((int adr));
static void WrDSPadr PARG((int adr, long data));

/* flags set on initialization ... */
static 	int lofiOn = 0;	/* flag that lofi believed booted */
static 	float usPerFrame = 0;	/* sample rate reciprocal, for pausing */
static 	int dspBufSiz = 0;	/* number of frames in dsp buffer */
static 	int dspHopSiz = 0;	/* how many samples consumed at 1ce by dsp */
/* pointers to dsp memory mapped in host address space */
static	unsigned long	*playBuf;
static	unsigned long	*playPosPtr;
static	unsigned long	*recBuf;
static	unsigned long	*recPosPtr;

#define SLOP_US		1000	/* how much to underwait */

static void WaitForLofiSampFrms(frames)	/* idle for dsp to clear <frames> */
    int 	frames;
    {
static struct 	timeval timeout;	/* static to avoid alloc time */
    float uS;

    timeout.tv_sec = 0L;		/* needn't be repeated */
    uS = frames * usPerFrame;
    if(uS < SLOP_US)
	return;
    else
	timeout.tv_usec = uS - SLOP_US;
    select(0,NULL,NULL,NULL,&timeout);
    }

static void timeout(s)
    char *s;
    {
    fprintf(stderr,"LoFi player: timeout at %s\n",s);
    exit(1);
    }

static int ReadDspPos()		/* returns frame index of latest used sample */
    {				/* WILL EQUAL blocksize - 1.. ? */
    long	dsps_next;
    int 	pos;

    if( (dsps_next = ((*playPosPtr)>>8L)/LOFI_CHANS)==0 )
	pos = dspBufSiz - 1;	/* dsp at start -> whole buffer read */
    else
	pos = dsps_next - 1;	/* else read up to sample preceding.. */
    return pos;
    }

static void WaitForDspPosAt(ourpos, posval)	/* block until dsp reaches point in buf */
    int 	ourpos;		/* where we are now */
    int 	posval;		/* minimum place for dsp pointer */
    {
    int 	dbg_count;
    int 	dsp_pos, dsp_next;

    dbg_count = 0;
    while(   ((dsp_pos = ReadDspPos()) < posval
	      && dsp_pos >= ourpos)
	  || ((dsp_next = (dsp_pos+dspHopSiz)%dspBufSiz) < posval
	      && dsp_next >= ourpos)
	  && dbg_count < MAXDBGWAIT )
	{
	WaitForLofiSampFrms((posval - dsp_pos + dspBufSiz)%dspBufSiz);
	++dbg_count;
	}
    if(dbg_count >= MAXDBGWAIT)
	timeout("waiting for buffer to open");
    }

static long RdDSPadr(adr)		/* read long from DSP valid address */
    int	adr;
    {
    return( (*(LoFiMap(RRAM, adr, 0)))>>8L );
    }

static void WrDSPadr(adr, data)
    int adr;
    long data;
    {
    *(LoFiMap(RRAM, adr, 1)) = data<<8L;
    }

int MyBootLoFi(srate)
    float srate;
    {
    long timeout;
    char *bootFile;
    char bootPath[128];
    char *loddir, *getenv();

    if(lofiOn)	{
	/* return 1;	/* lofi already on */
    } else {
	if((void *)LoFiOpen("/dev/lofi") == NULL)
	{ fprintf(stderr,"Cannot open LoFi\n");
	  return(0);	}
    }
    if((loddir = getenv("LODDIR")) == NULL)
	{
/*	fprintf(stderr,"no LODDIR env variable\n");	return(0);	*/
	loddir = "/usr/local/lib/lod";	/* give it a go if no env */
	}
    /* assume only decaudio supported.  Choose poss sampling rate */
    if(srate > 26000)        { bootFile = "play44ext.lod"; 
			       usPerFrame = 22.676; }
    else if(srate > 18000)   { bootFile = "play22ext.lod"; 
			       usPerFrame = 45.351; }
    else 		     { bootFile = "play11ext.lod"; 
			       usPerFrame = 90.703; }

    sprintf(bootPath,"%s/%s",loddir,bootFile);
/*    fprintf(stderr,"bootfile: %s\n",bootPath);	/* */
    {
      FILE *f;
      if( (f=fopen(bootPath, "r"))==NULL)  {
        fprintf(stderr, "BootLofi: cannot find LOD file '%s'\n  Check your LODDIR environment variable\n", bootPath);
	return 0;
      } else {
        fclose(f);
      }
    }

    LoFiSetCSR(FED, 0);		/* disable DSP */
    dspLoad(bootPath);		/* load code */
    WrDSPadr(ALIVEF, 0L);	/* mark DSP as unbooted */
    WrDSPadr(SRATE, (long)srate);   /* notify requested sample rate */
    LoFiSetCSR(FED, 1);		/* reboot DSP */

    timeout = 1000;
    while( RdDSPadr(ALIVEF) == 0L && --timeout)
	WaitForLofiSampFrms((int)1000/usPerFrame);	/* approx 1ms pause */
    if(timeout == 0L)
	{
	fprintf(stderr,"Timeout waiting for DSP to boot\n");
	return(0);
	}
    /* buffer details - only set these once at boot */
    playBuf = LoFiMap(RRAM,(int)RdDSPadr(HOBBASE),1);
    recBuf  = LoFiMap(RRAM,(int)RdDSPadr(HIBBASE),1);
    dspBufSiz = (int)RdDSPadr(HBSIZE)/LOFI_CHANS;
    dspHopSiz = (int)RdDSPadr(HBBLCK)/LOFI_CHANS;  
    playPosPtr = LoFiMap(RRAM, HIBPNTR, 0);
    recPosPtr = LoFiMap(RRAM, HOBPNTR, 0); /* addresses to read ptrs from */
    /*    fprintf(stderr, "bufsize = %ld frames\n", dspBufSiz);  */
    lofiOn = 1;
    return 1;
    }

static int AUSampleSizeOf(int format)
{   /* return number of bytes per sample in given fmt */
    return(format & AU_BYTE_SIZE_MASK);
}	/* copied from SFSampleSizeOf */

AUSTRUCT *AUOpen(int format, double srate, int chans, long desbuf, int mode)
{  /* open a new audio IO channel.  Define data format, #channels and sampling
      rate. Request that output buffer be desbuf frames long.  
      mode is AIO_{INPUT,OUTPUT} */
    AUSTRUCT *au = NULL;
    int	     err = 0;

    DBGFPRINTF((stderr, "AUOpen(fmt %d, sr %.0lf, ch %d, desb %ld, mo %d)\n",
		format, srate, chans, desbuf, mode));
    /* check arguments */
    if(format != AFMT_SHORT)  {	/* just shorts for lofi */
	fprintf(stderr, "AUOpen: format %d not supported\n", format);
	++err;
    }
    if(srate < 8000 || srate > 48000 /* || ... */)  {
	fprintf(stderr, "AUOpen: srate %.0f not supported\n", srate);
	++err;
    }
    if(chans < 1 || chans > 2 /* || ... */)  {
	fprintf(stderr, "AUOpen: chans %d not supported\n", chans);
	++err;
    }
    if(mode != AIO_INPUT && mode != AIO_OUTPUT)  {
	fprintf(stderr, "AUOpen: mode %d not supported\n", mode);
	++err;
    }
    /* not sure what to do with desbuf .. */
    if(desbuf <= 0)	
	desbuf = srate * DFLT_BUFTIME;

    if(err == 0)  {	/* actual SGI-specific port initialization */
	long width = AUSampleSizeOf(format);

	/* open the audio */
	if(mode == AIO_INPUT) {
	    fprintf(stderr, "AUOpen: no record on DECaudio: sorry\n");
	} else {
	}
	
	if(MyBootLoFi((float)srate)==0)	{
	    return 0;    /* LoFi boot failed */
	}

	/* allocate the descriptor */
	au = TMMALLOC(AUSTRUCT, 1, "AUOpen");
	au->mode   = mode;	au->chans = chans;
	au->format = format;	au->port  = (void *)-1;
	au->ourPos = -1;	/* indicate that it's the first pass */

	if(au->port == NULL) {
	    /* error initializing hardware */
	    free(au);
	    return NULL;
	}
    }
    return au;
}

long	AUWrite(AUSTRUCT *au, void *buf, long frames)
{   /* write some sound to the open audio IO channel, per description */
    short 	*src = buf;		/* 16bit samples (pairs if stereo) */
    long	siz = frames;	/* number of sample-frames in src */
    int 	max_at_once;	/* maximum contiguous space at present */
    int 	to_write_now;	/* how many frames we will do this pass */
    int 	left_to_write;	/* counter for copying samples */
    int 	reqd_dsp_pos;	/* frame index we want from dsp */
    unsigned long 	*playBufPtr;	/* where to put samples */

    assert((au->mode) == AIO_OUTPUT);
    if(au->ourPos == -1)  {	/* first time we are called after init */
	WaitForDspPosAt(dspHopSiz, dspBufSiz - 1);	
	/* start whole thing open */
	au->ourPos = 0;
    }

    /* ourPos records the next element of the buffer we will write to */
    max_at_once = dspBufSiz - au->ourPos;	/* max we can write at 1ce */
    if(siz > max_at_once)
	to_write_now = max_at_once;
    else
	to_write_now = siz;
    if(to_write_now > dspBufSiz - 2*dspHopSiz)
        to_write_now = dspBufSiz - 2*dspHopSiz;	/* biggest gap */
    reqd_dsp_pos = au->ourPos + to_write_now - 1;   /* minus one is correct */
    /* wait for enough space to open up in LOFI buffer to write all at 1ce */
    WaitForDspPosAt(au->ourPos, reqd_dsp_pos);
    /* now have at least to_write_now spaces open in the buffer */
    playBufPtr = playBuf + LOFI_CHANS*au->ourPos;

    left_to_write = to_write_now;
    while(left_to_write--)  {	/* copy them all */
	*playBufPtr++ = ((long)*src)<<16;
	*playBufPtr++ = ((long)*(src+au->chans-1))<<16;
	src += au->chans;
    }
    if( (au->ourPos += to_write_now) == dspBufSiz)
	au->ourPos = 0;
    siz -= to_write_now;	/* while we still know how many */
    if(siz > 0)  {
	AUWrite(au, src, siz);
    }
    return frames;	/* presumably */
}

long	AURead(AUSTRUCT *au, void *buf, long frames)
{   /* read some sound from the open audio IO channel, per description */
    assert(au->mode == AIO_INPUT);
    fprintf(stderr, "AURead: sorry; no record on DECaudio\n");
    return 0;
}

long AUBufConts(AUSTRUCT *au)
{   /* see how many sample frames are currently in audio buffer.
       (can be used for synchronization, timing etc). */
    return (au->ourPos - ReadDspPos() + dspBufSiz) % dspBufSiz;
}

void	AUClose(AUSTRUCT *au, int now)
{  /* close an audio IO channel. if <now> is nonzero, stop immediately
      else play out any buffered samples */
/*
    if(now == 0
       && au->mode == AIO_OUTPUT)
	while(AUBufConts(au)>0)
	    sginap(1);
 */
    LoFiClose();
    lofiOn = 0;
    free(au);
}

