/***************************************************************\
*	sndfsd2.c					MACINTOSH						*
*	Extra functions for SoundDesignerII version of sndf			*
*	THIS FILE IS CONDITIONALLY INCLUDED IN sndf.c				*
*	12aug90 dpwe												*
\***************************************************************/

/* exported file type indicator */
long sndfType = (long)'Sd2f';
#ifdef AUDIOMEDIA
long sndfCrea = (long)'Sd2b';
#else /* not AUDIOMEDIA -> good ole' soundDesigner */
long sndfCrea = (long)'Sd2a';
#endif /* AUDIOMEDIA */
char *sndfExtn = ".sd2";		/* file extension, if used */

/* subsidiary extern func protos */
#include <pascal.h>		/* for PtoCstr */
#include "macsndf.h"
#include "sndffname.c"	/* define AddFname, FindFname etc */

/* private type declarations */

/* static variables */

/* static function prototypes */

/* ----------------------- implementations ------------------------ */

int SFReadHdr(FILE *file, SFSTRUCT *snd, int infoBmax)
/* utility to sample the header of an opened file, at most infoBmax info byts */
    {
    int 	chans,dsize,infoSize;
    long	len;
    float	srate;
    char	*info;
    char	*fname;
    int 	err,i;

	if(infoBmax < SFDFLTBYTS)	infoBmax = SFDFLTBYTS;
	infoBmax = infoBmax&-4;		/* round down to multiple of four */
	infoSize = infoBmax;
	if( (fname = FindFname(file)) == NULL)
		return(SFerror = SFE_NOPEN);		/* couldn't find a pathname for this */
	CtoPstr(fname);
	info = snd->info;			/* copy info directly into buffer */
	err = MacReadHeader(fname,0,&chans,&srate,&dsize,info,&infoSize);
	/* will only read into info for infoSize byts, updates infoSize to avai bts */

/*	len = MacFileLen(fname,0);
	if(len < 0)					/* couldn't read file length */
/*		printf("SFRdHdr: len read failed\n");
 */
 	len = file->len;			/* cheat by using THINK_C's internal record... */

	PtoCstr(fname);
	if(err&7)					/* just bottom 3 bits critical - optional info */
		return(SFerror = SFE_NSF);		/* reading mac header failed */
	if(infoSize < SFDFLTBYTS)	
		infoSize = SFDFLTBYTS;
	if(infoBmax > infoSize) 	infoBmax = infoSize;	/* actual bts in buf */
    snd->magic = SFMAGIC;
   	snd->headBsize = sizeof(SFSTRUCT) + infoSize - SFDFLTBYTS;
	snd->dataBsize = len;	snd->channels = chans;	snd->samplingRate = srate;
	if(dsize == 1)		snd->format = SFMT_CHAR;
	else if(dsize == 4)	snd->format = SFMT_FLOAT;
	else 				snd->format = SFMT_SHORT;
    return(SFerror = SFE_OK);
    }

int SFRdXInfo(FILE *file, SFSTRUCT *psnd, int done)
/* fetch any extra info bytes needed in view of read header
    Passes open file handle, existing sound struct
    already read 'done' bytes; rest inferred from structure */
	{
    int 	chans,dsize,infoSize;
    long	len;
    float	srate;
    char	*info;
    char	*fname;
    int 	err,i;
    int 	rem;
	
	if( (fname = FindFname(file)) == NULL)
		return(SFerror = SFE_NOPEN);		/* couldn't find a pathname for this */
	CtoPstr(fname);
	infoSize = SFInfoSize(psnd); /* reread whole of info */
	err = MacReadHeader(fname,0,&chans,&srate,&dsize,psnd->info,&infoSize);
	PtoCstr(fname);
	if(err)
		return(SFerror = SFE_NOPEN);		/* reading mac header failed */
	else
		return(SFerror = SFE_OK);
	}

int SFWriteHdr(FILE *file, SFSTRUCT *snd, int infoBmax)
/* 	Utility to write out header to open file
	infoBmax optionally limits the number of info chars to write.
	infoBmax = 0 => write out all bytes implied by .headBsize field 
	infoBmax >= SFDFLTBYS => write out only infoBmax infobytes */
    {
    int 	chans,dsize,infoSize;
    float	srate;
    char	*info;
    char	*fname;
    int 	err,i;

    if(snd->magic != SFMAGIC)
		return(SFerror = SFE_NSF);
	if(infoBmax >= SFDFLTBYTS && infoBmax < SFInfoSize(snd) )
		infoSize = infoBmax;
	else
		infoSize = SFInfoSize(snd);
	chans = snd->channels;	srate = snd->samplingRate;
	if(snd->format == SFMT_CHAR || snd->format == SFMT_ALAW ||
			 snd->format == SFMT_ULAW)
		dsize = 1;
	else if(snd->format == SFMT_LONG || snd->format == SFMT_FLOAT)
		dsize = 4;
	else dsize = 2;		/* default short */
	info = snd->info;
	if( (fname = FindFname(file)) == NULL)
		return(SFerror = SFE_NOPEN);		/* couldn't find a pathname for this */
	CtoPstr(fname);
	err=MacAddHeader(fname,0/*vref*/,chans,srate,dsize,info,infoSize);
	MacSetTypCreAs(fname,0/*vref*/, sndfType, sndfCrea);		/* make an Sd2 file */
	PtoCstr(fname);
	if(err)
		return(SFerror = SFE_WRERR);
	else
	    return(SFerror = SFE_OK);
    }

static long SFHeDaLen(FILE *file) /* find header length for a file */
	{
	return(0L);		/* macintosh headers occupy no file data space */
	}

static long SFLastSampTell(file)	/* return last valid ftell for smps */
    FILE	*file;
    {
    return (0L);	/* punt on this for here - just use EOF */
    }

static void FixupSamples(ptr, format, nitems, flag)
    void  *ptr;
    int   format;
    long  nitems;
    int   flag;		/* READ_FLAG or WRITE_FLAG indicates direction */
    {
    }	/* always machine order */
